"use strict";

exports.__esModule = true;
exports["default"] = exports.css = void 0;

var _parser = require("@chakra-ui/parser");

var _utils = require("@chakra-ui/utils");

var _css = require("./css.utils");

var css = function css(styleObject) {
  return function (props) {
    var theme = (0, _css.determineTheme)(props);
    var computedStyles = {};
    var style = (0, _utils.runIfFn)(styleObject, theme);
    var styles = (0, _parser.parsePseudo)(style);
    var queries = (0, _parser.getMediaQuery)(theme.breakpoints);

    var responsive = function responsive(prop, values, config) {
      return (0, _parser.assignArray)({
        mediaQueries: queries.asArray,
        prop: prop,
        values: values,
        transform: function transform(value) {
          return (0, _css.transformWithConfig)(theme, value, config);
        }
      });
    };

    var _loop = function _loop(key) {
      var valueOrFn = styles[key];
      var value = (0, _utils.runIfFn)(valueOrFn, theme);
      var config = _parser.parser.config[key];

      if (key === "apply") {
        var _style = css((0, _utils.get)(theme, value))(theme);

        computedStyles = (0, _utils.merge)(computedStyles, _style);
        return "continue";
      }

      if ((0, _utils.isObject)(value)) {
        var _style2 = (0, _css.transformWithConfig)(theme, value, config);

        computedStyles[key] = css(_style2)(theme);
        return "continue";
      }

      if ((0, _utils.isArray)(value)) {
        if (config == null ? void 0 : config.properties) {
          config.properties.forEach(function (prop) {
            var style = responsive(prop, value, config);
            computedStyles = (0, _utils.merge)(computedStyles, style);
          });
          return "continue";
        }

        if (config == null ? void 0 : config.property) {
          var _style3 = responsive(config.property, value, config);

          computedStyles = (0, _utils.merge)(computedStyles, _style3);
          return "continue";
        }

        if (config === true) {
          var _style4 = responsive(key, value, config);

          computedStyles = (0, _utils.merge)(computedStyles, _style4);
          return "continue";
        }
        /**
         * If no config exists and `value` is still an array
         * run it through `css` in-case it contains style props
         */


        if ((0, _utils.isArray)(value)) {
          var val = value.map(function (v) {
            return css(v)(theme);
          });

          var _style5 = responsive(key, val, config);

          computedStyles = (0, _utils.merge)(computedStyles, _style5);
          return "continue";
        }
      }

      if (config == null ? void 0 : config.properties) {
        config.properties.forEach(function (prop) {
          computedStyles[prop] = (0, _css.transformWithConfig)(theme, value, config);
        });
        return "continue";
      }

      if (config == null ? void 0 : config.property) {
        var _style6 = (0, _css.transformWithConfig)(theme, value, config);

        computedStyles[config.property] = _style6;
        return "continue";
      }

      computedStyles[key] = (0, _css.transformWithConfig)(theme, value, config);
    };

    for (var key in styles) {
      var _ret = _loop(key);

      if (_ret === "continue") continue;
    }

    return computedStyles;
  };
};

exports.css = css;
var _default = css;
exports["default"] = _default;
//# sourceMappingURL=css.js.map