import { assignArray, getMediaQuery, parsePseudo, parser } from "@chakra-ui/parser";
import { get, isArray, isObject, merge, runIfFn } from "@chakra-ui/utils";
import { determineTheme, transformWithConfig as tx } from "./css.utils";
export var css = styleObject => props => {
  var theme = determineTheme(props);
  var computedStyles = {};
  var style = runIfFn(styleObject, theme);
  var styles = parsePseudo(style);
  var queries = getMediaQuery(theme.breakpoints);

  var responsive = (prop, values, config) => {
    return assignArray({
      mediaQueries: queries.asArray,
      prop,
      values,
      transform: function transform(value) {
        return tx(theme, value, config);
      }
    });
  };

  var _loop = function _loop(key) {
    var valueOrFn = styles[key];
    var value = runIfFn(valueOrFn, theme);
    var config = parser.config[key];

    if (key === "apply") {
      var _style = css(get(theme, value))(theme);

      computedStyles = merge(computedStyles, _style);
      return "continue";
    }

    if (isObject(value)) {
      var _style2 = tx(theme, value, config);

      computedStyles[key] = css(_style2)(theme);
      return "continue";
    }

    if (isArray(value)) {
      if (config == null ? void 0 : config.properties) {
        config.properties.forEach(prop => {
          var style = responsive(prop, value, config);
          computedStyles = merge(computedStyles, style);
        });
        return "continue";
      }

      if (config == null ? void 0 : config.property) {
        var _style3 = responsive(config.property, value, config);

        computedStyles = merge(computedStyles, _style3);
        return "continue";
      }

      if (config === true) {
        var _style4 = responsive(key, value, config);

        computedStyles = merge(computedStyles, _style4);
        return "continue";
      }
      /**
       * If no config exists and `value` is still an array
       * run it through `css` in-case it contains style props
       */


      if (isArray(value)) {
        var val = value.map(v => css(v)(theme));

        var _style5 = responsive(key, val, config);

        computedStyles = merge(computedStyles, _style5);
        return "continue";
      }
    }

    if (config == null ? void 0 : config.properties) {
      config.properties.forEach(prop => {
        computedStyles[prop] = tx(theme, value, config);
      });
      return "continue";
    }

    if (config == null ? void 0 : config.property) {
      var _style6 = tx(theme, value, config);

      computedStyles[config.property] = _style6;
      return "continue";
    }

    computedStyles[key] = tx(theme, value, config);
  };

  for (var key in styles) {
    var _ret = _loop(key);

    if (_ret === "continue") continue;
  }

  return computedStyles;
};
export default css;
//# sourceMappingURL=css.js.map